using System;
using System.Runtime.InteropServices;

namespace LibVlc
{
	public class Media : Object
	{
		[DllImport("vlc")]
		static extern IntPtr libvlc_media_new_location(
			IntPtr instance,
			[MarshalAs(UnmanagedType.LPStr)] string name);
		[DllImport("vlc")]
		static extern IntPtr libvlc_media_new_path(
			IntPtr instance,
			[MarshalAs(UnmanagedType.LPStr)] string path);
		[DllImport("vlc")]
		static extern IntPtr libvlc_media_new_fd(IntPtr instance,int fd);
		[DllImport("vlc")]
		static extern IntPtr libvlc_media_new_as_node(
			IntPtr instance,
			[MarshalAs(UnmanagedType.LPStr)] string node_name);
		[DllImport("vlc")]
		static extern void libvlc_media_add_option(
			IntPtr media,
			[MarshalAs(UnmanagedType.LPStr)] string option);
		[DllImport("vlc")]
		static extern void libvlc_media_add_option_flag(
			IntPtr media,
			[MarshalAs(UnmanagedType.LPStr)] string option,
			uint flags);
		[DllImport("vlc")]
		static extern IntPtr libvlc_media_get_mrl(IntPtr media);
		[DllImport("vlc")]
		static extern IntPtr libvlc_media_duplicate(IntPtr media);
		[DllImport("vlc")]
		static extern IntPtr libvlc_media_get_meta(IntPtr media, Meta meta);
		[DllImport("vlc")]
		static extern void libvlc_media_set_meta(IntPtr media, Meta meta, 
		                          [MarshalAs(UnmanagedType.LPStr)]string val);
		[DllImport("vlc")]
		static extern int libvlc_media_save_meta(IntPtr media);
		[DllImport("vlc")]
		static extern int libvlc_media_get_state(IntPtr media);
		[DllImport("vlc")]
		static extern int libvlc_media_get_stats(IntPtr media, IntPtr stats);
		[DllImport("vlc")]
		static extern IntPtr libvlc_media_subitems(IntPtr media);
		[DllImport("vlc")]
		static extern IntPtr libvlc_media_event_manager(IntPtr media);
		[DllImport("vlc")]
		static extern void libvlc_media_parse(IntPtr media);
		[DllImport("vlc")]
		static extern void libvlc_media_parse_async(IntPtr media);
		[DllImport("vlc")]
		static extern int libvlc_media_is_parsed(IntPtr media);
		[DllImport("vlc")]
		static extern long libvlc_media_get_duration(IntPtr media);
		[DllImport("vlc")]
		static extern int libvlc_media_get_tracks_info(IntPtr media, [MarshalAs(UnmanagedType.LPArray)]out TrackInfo[] track_infos);
		public Media (IntPtr raw) : base(raw){}
		public Media (Instance instance, Uri location)
			: base(libvlc_media_new_location (instance.Handle,location.OriginalString))
		{}
		public Media(Instance instance, string path)
			: base(libvlc_media_new_path (instance.Handle,path))
		{}
		public Media(Instance instance, int fd)
			: base(libvlc_media_new_fd (instance.Handle,fd))
		{}
		public static Media AsNode(Instance instance, string node_name){
			return new Media(libvlc_media_new_as_node(instance.Handle,node_name));
		}
		public void AddOption(string option){
			libvlc_media_add_option (Handle,option);
		}
		public void AddOption(string option, OptionType flags){
			libvlc_media_add_option_flag (Handle, option, (uint)flags);
		}
		public string Mrl{
			get{return Marshal.PtrToStringAuto (libvlc_media_get_mrl(Handle));}
		}
		public Media Duplicate(){
			return new Media(libvlc_media_duplicate (Handle));
		}
		public void Parse(){libvlc_media_parse (Handle);}
		public void ParseAync(){libvlc_media_parse_async (Handle);}
		public bool IsParsed(){return (libvlc_media_is_parsed (Handle)==1)?true:false;}
		public string this[Meta meta]{
			get{return Marshal.PtrToStringAuto (libvlc_media_get_meta (Handle,meta));}
			set{libvlc_media_set_meta (Handle,meta,value);}
		}
		public bool SaveMeta(){
			return (libvlc_media_save_meta (Handle)==1)?true:false;
		}
		public State GetState(){
			return (State)libvlc_media_get_state (Handle);
		}
		public bool GetStats(out MediaStats stats){
			stats = new MediaStats();
			IntPtr ptr = Handle;
			Marshal.StructureToPtr (stats,ptr,false);
			bool b = (libvlc_media_get_stats (Handle,ptr) == 1)?true:false;
			stats = (MediaStats)Marshal.PtrToStructure(ptr,typeof(MediaStats));
			return b;
		}
		public long Duration{get{return libvlc_media_get_duration (Handle);}}
		public MediaList SubItems{get{
				return new MediaList(libvlc_media_subitems (Handle));
			}}
		public LibVlc.EventManager EventManager{
			get{return new LibVlc.EventManager(libvlc_media_event_manager (Handle));}
		}
		public int GetTracksInfo(out TrackInfo[] infos){
			return libvlc_media_get_tracks_info (Handle,out infos);
		}
	}
}

